#!/bin/bash

set -eux

status=0

SCRIPT_DIR=$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )
cd $SCRIPT_DIR

pkill -KILL postgres || true
pkill -KILL minio || true
rm -rf minio env pgdata pgdata_backup logfile backup_logfile

mkdir minio
minio server minio --address :9000 --console-address :9001 &

while ! nc -z localhost 9000; do
  sleep 0.1
done

mc alias set 'local' 'http://127.0.0.1:9000' 'minioadmin' 'minioadmin'
# create bucket
mc mb local/test

mkdir env
# echo 'DEVEL' >env/WALG_LOG_LEVEL
echo 'NORMAL' >env/WALG_LOG_LEVEL
# echo 'true' >env/WALG_USE_WAL_DELTA
echo 'brotli' >env/WALG_COMPRESSION_METHOD
echo '6' >env/WALG_DELTA_MAX_STEPS

echo 'minioadmin' >env/AWS_SECRET_ACCESS_KEY
echo 'minioadmin' >env/AWS_ACCESS_KEY_ID
echo 's3://test' >env/WALG_S3_PREFIX
echo 'http://127.0.0.1:9000' >env/AWS_ENDPOINT
echo 'true' >env/AWS_S3_FORCE_PATH_STYLE
echo 'us-east-1' >env/AWS_REGION

echo 'localhost' >env/PGHOST
echo '5432' >env/PGPORT
echo "$USER" >env/PGUSER
echo "postgres" >env/PGDATABASE

initdb --no-locale -D pgdata

echo "archive_mode = yes" >> pgdata/postgresql.conf
echo "archive_command = 'envdir $PWD/env wal-g wal-push %p'" >> pgdata/postgresql.conf
echo "archive_timeout = 60" >> pgdata/postgresql.conf
echo "restore_command = 'envdir $PWD/env wal-g wal-fetch \"%f\" \"%p\"'" >> pgdata/postgresql.conf
echo "shared_preload_libraries = 'orioledb'" >> pgdata/postgresql.conf
echo "orioledb.main_buffers = 512MB" >> pgdata/postgresql.conf
echo "orioledb.undo_buffers = 256MB" >> pgdata/postgresql.conf
echo "max_wal_size = 8GB" >> pgdata/postgresql.conf

# start server
pg_ctl -D pgdata start &

while ! pg_isready -d "postgres"; do
  sleep 0.1
done

# check server
psql -d postgres -c "CREATE EXTENSION orioledb;"
psql -d postgres -c "CREATE TABLE o_test(a int) USING orioledb;"
# psql -d postgres -c "CREATE TABLE o_test(a int) USING heap; "
psql -d postgres -c "INSERT INTO o_test VALUES (1), (5), (2);"
psql -d postgres -c "TABLE o_test;"

# to send more wal files
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "SELECT COUNT(*) FROM o_test;"
psql -d postgres -c "SELECT pg_switch_wal();"

echo 'DEVEL' >env/WALG_LOG_LEVEL

# envdir env wal-g backup-push pgdata
envdir env wal-g backup-push --with-orioledb pgdata

# to send more wal files
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "SELECT COUNT(*) FROM o_test;"
psql -d postgres -c "SELECT pg_switch_wal();"

# psql -d postgres -c "SELECT orioledb_tbl_structure('o_test'::regclass);"

# envdir env wal-g backup-push pgdata
envdir env wal-g backup-push --with-orioledb pgdata

# to send more wal files
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "SELECT COUNT(*) FROM o_test;"
psql -d postgres -c "SELECT pg_switch_wal();"

LAST_BACKUP=$(envdir env wal-g backup-list 2>/dev/null | tail -1 | cut -f1 -d' ')
envdir env wal-g backup-fetch pgdata_backup $LAST_BACKUP
touch pgdata_backup/standby.signal
echo "port=5433" >> pgdata_backup/postgresql.conf
echo "recovery_target = 'immediate'" >> pgdata_backup/postgresql.conf

pg_ctl -D pgdata_backup start &

while ! pg_isready -p 5433 -d "postgres"; do
  sleep 0.1
done

psql -p 5433 -d postgres -c "SELECT COUNT(*) FROM o_test;"

# to send more wal files
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "INSERT INTO o_test SELECT generate_series(1, 10000);"
psql -d postgres -c "SELECT COUNT(*) FROM o_test;"
psql -d postgres -c "SELECT pg_switch_wal();"

psql -p 5433 -d postgres -c "SELECT COUNT(*) FROM o_test;"

pg_ctl -D pgdata_backup -l backup_logfile stop
echo "recovery_target = ''" >> pgdata_backup/postgresql.conf
pg_ctl -D pgdata_backup start &

while ! pg_isready -p 5433 -d "postgres"; do
  sleep 0.1
done

psql -d postgres -c "SELECT pg_switch_wal();"
lsn_to_int () {
	echo $1 | gawk --non-decimal-data 'BEGIN {OFS = FS} {split($0,a,"/"); high=lshift(sprintf("%d", "0x" a[1]), 32); low=sprintf("%d", "0x" a[2]); print high+low}'
}
WAIT_LSN=$(lsn_to_int $(psql -d postgres -t -c "select pg_catalog.pg_current_wal_lsn()::text;" | head -1))
REPLICA_LSN=$(lsn_to_int $(psql -p 5433 -d postgres -t -c "SELECT pg_catalog.pg_last_wal_replay_lsn();" | head -1))
while [ $WAIT_LSN -gt $REPLICA_LSN ]; do
  	sleep 0.1
	REPLICA_LSN=$(lsn_to_int $(psql -p 5433 -d postgres -t -c "SELECT pg_catalog.pg_last_wal_replay_lsn();" | head -1))
done
SYNCHRONIZED=$(psql -p 5433 -d postgres -t -c "SELECT orioledb_recovery_synchronized();" | head -1 | tr -d ' ')
while [ $SYNCHRONIZED == 'f' ]; do
  	sleep 0.1
	SYNCHRONIZED=$(psql -p 5433 -d postgres -t -c "SELECT orioledb_recovery_synchronized();" | head -1 | tr -d ' ')
done

psql -p 5433 -d postgres -c "SELECT COUNT(*) FROM o_test;"

pkill -KILL postgres
pkill -KILL minio
